"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const child_process_1 = require("child_process");
const util_1 = require("util");
const fs_1 = require("fs");
const pgPromise = require("pg-promise");
const glob = require("glob");
const concatFiles = require("concat-files");
const readline = require("readline");
const globAsync = util_1.promisify(glob);
const concatAsync = util_1.promisify(concatFiles);
const unlinkAsync = util_1.promisify(fs_1.unlink);
const renameAsync = util_1.promisify(fs_1.rename);
const existsAsync = util_1.promisify(fs_1.exists);
const execAsync = util_1.promisify(child_process_1.exec);
const pgp = pgPromise({ promiseLib: Promise });
const unlinkIfExists = (path) => unlinkAsync(path)
    .catch((err) => {
    if (err.code === 'ENOENT')
        console.log(`${path} does not exists`);
    else
        throw err;
});
exports.ask = (question, log) => {
    const rl = readline.createInterface({
        input: process.stdin,
        output: process.stdout,
    });
    return new Promise((resolve) => {
        rl.question(question + ' [y,n] ', (answer) => {
            if (answer.indexOf('y') === 0) {
                resolve(true);
            }
            else if (answer.indexOf('n') === 0) {
                resolve(false);
            }
            else {
                log.always('Could not parse input, please choose yes or no');
                return exports.ask(question, log);
            }
            rl.close();
        });
    });
};
exports.initCompareByConcat = (program, log) => {
    return existsAsync(program.compare)
        .then((compareExists) => {
        if (!compareExists) {
            log.always('File for comparing combined sql files to does not exists: ' + program.compare);
            return exports.ask('Create file from combined sql files?', log)
                .then(accepted => {
                if (accepted) {
                    return exports.concat(program, program.compare, false, log);
                }
            })
                .then(() => {
                log.always('Please restart');
                process.exit(0);
            })
                .catch(console.error);
        }
    });
};
exports.concat = (program, output, unlinkBefore, log) => {
    return Promise.all([
        globAsync(program.source),
        unlinkBefore ? unlinkIfExists(output) : Promise.resolve(),
    ])
        .then((data) => {
        const fileNames = data[0];
        log.verbose('processing files: ', fileNames);
        return concatAsync(fileNames, output)
            .then((res) => {
            log.verbose('concatenated files and saved into: ', output);
            return res;
        });
    });
};
exports.compare = (program, log) => {
    const apgDiffPath = path_1.resolve(__dirname + '/../../vendor/apgdiff-2.5next/apgdiff-2.5.0-SNAPSHOT.jar');
    const parameter = '--ignore-start-with --add-transaction ' + program.compare + ' ' + program.tmp + ' | tee ' + program.diff;
    return execAsync('java -jar ' + apgDiffPath + ' ' + parameter)
        .then(({ stdout, stderr }) => {
        if (stderr)
            throw new Error(stderr);
        log.verbose('Schema Diff: ' + stdout);
        return stdout;
    });
};
const connections = {};
const getConnection = (connectString) => {
    if (!connections.hasOwnProperty(connectString)) {
        connections[connectString] = pgp(connectString);
    }
    return connections[connectString];
};
exports.upgrade = (program, diff, log) => {
    const db = getConnection(program.connection);
    return db.query(diff)
        .then(function (result) {
        log.always('Postgres result: ', result);
    });
};
exports.moveFiles = (program, log) => {
    log.always('Deleting ' + program.compare);
    return unlinkIfExists(program.compare)
        .then(() => {
        log.always('Moving ' + program.tmp + ' to ' + program.compare);
        return renameAsync(program.tmp, program.compare);
    });
};
